# Crunchy BackRest
#
# License: MIT
# Portions Copyright (c) 2015-2018, The PostgreSQL Global Development Group
# Portions Copyright (c) 2013-2018, David Steele
# Authors: John Harvey <john.harvey@crunchydata.com>, Adam Brightwell <adam.brightwell@crunchydata.com>

# Global Definitions

%define SCRIPT_NAME crunchy-backrest
%define PERL_LIB_DIR /usr/share/perl5
%define C_LIB_DIR /usr/local/lib64/perl5

# RPM Spec File
#
# crunchybackrest_specfile_version must be provided as an argument to 'rpmbuild' using --define.
#
# Example:
#  rpmbuild -ba --define 'crunchybackrest_specfile_version 0.77' crunchy-backrest.spec
#

Name:		%{SCRIPT_NAME}
Version:	%{crunchybackrest_specfile_version}
Release:	%{crunchybackrest_specfile_release}%{?dist}
Vendor:		Crunchy Data Solutions, Inc.
Summary:	Crunchy Backup Management Tool for PostgreSQL

Group:		Applications/Databases
License:	MIT
URL:		http://www.crunchydata.com
Source0:        %{name}-%{version}.tar.gz
Source1:	pgbackrest.conf
Source2:	pgbackrest.logrotate
BuildRoot:	%(mktemp -ud %{_tmppath}/%{name}-%{version}-%{release}-XXXXXX)
BuildArch:	x86_64
BuildRequires:	perl-devel openssl-devel perl-ExtUtils-Embed zlib-devel libxml2-devel bzip2-devel
Requires:	libxml2 bzip2-libs
# New requirements as of 2.16
BuildRequires:	postgresql13-devel
Requires:	postgresql13-libs

%if 0%{?rhel} && 0%{?rhel} == 8
Requires:       lz4-libs
BuildRequires:  lz4-devel
%endif
%if 0%{?rhel} && 0%{?rhel} == 7
Requires:       lz4
BuildRequires:  lz4-devel
%endif

Provides:	pgbackrest = %{version}

%description
Crunchy Backup Management Tool for PostgreSQL

%prep
%setup -q

%build
cd src
export CPPFLAGS='-I %{pginstdir}/include'
export LDFLAGS='-L%{pginstdir}/lib'
%configure
%{__make}

%install
install -D -m 755 ${RPM_BUILD_DIR}/%{name}-%{version}/src/pgbackrest ${RPM_BUILD_ROOT}%{_bindir}/pgbackrest
install -D -m 644 %{SOURCE1} ${RPM_BUILD_ROOT}/etc/pgbackrest.conf

# Install logrotate file:
%{__install} -p -d %{buildroot}%{_sysconfdir}/logrotate.d
%{__install} -p -m 644 %{SOURCE2} %{buildroot}%{_sysconfdir}/logrotate.d/%{name}

%pre
# This takes arguments.  If "1" then it's an install, if "2" then it's an upgrade

if [ -f /etc/pgbackrest/pgbackrest.conf ]; then
    if [ ! -f /etc/pgbackrest.conf ] ; then
        # We now know that the user removed the default conf file and are using their own in the new location
        # Create a temporary file so I know to remove the default installed /etc/pgbackrest.conf file
        touch /etc/pgbackrest.rm.conf.crunchy
    fi
fi

if [ -f /etc/pgbackrest.conf ]; then
    # In the case of an upgrade or reinstall we can save off the current conf file to get the permissions off it in post
    %{__cp} -a /etc/pgbackrest.conf /etc/pgbackrest.conf.crunchy.tmp
fi 

%post
# This takes arguments.  If "1" then it's an install, if "2" then it's an upgrade

if [ -f /etc/pgbackrest.rm.conf.crunchy ]; then
    # We know the user wants to use the new config file and location so remove the default installed one
    rm -f /etc/pgbackrest.conf
    rm -f /etc/pgbackrest.rm.conf.crunchy
fi

if [ "$1" == "1" ]; then
    # INSTALL

    # If the /var/log/pgbackrest directory exists always use that ownership for the conf file
    if [ -d /var/log/pgbackrest ]; then
        echo "*** WARNING: "
        echo "    Using the ownership from existing /var/log/pgbackrest directory"
        echo "***"

       if [ -f /etc/pgbackrest.conf ]; then
           chown --reference=/var/log/pgbackrest /etc/pgbackrest.conf
           chgrp --reference=/var/log/pgbackrest /etc/pgbackrest.conf
       fi

       if [ ! -d /var/lib/pgbackrest ]; then
           mkdir -p /var/lib/pgbackrest
           chmod 750 /var/lib/pgbackrest
           chown --reference=/var/log/pgbackrest /var/lib/pgbackrest
           chgrp --reference=/var/log/pgbackrest /var/lib/pgbackrest
       fi

       if [ ! -d /var/spool/pgbackrest ]; then
           mkdir -p /var/spool/pgbackrest
           chmod 750 /var/spool/pgbackrest
           chown --reference=/var/log/pgbackrest /var/spool/pgbackrest
           chgrp --reference=/var/log/pgbackrest /var/spool/pgbackrest
       fi

    else
        mkdir -p /var/log/pgbackrest
        chmod 750 /var/log/pgbackrest
        mkdir -p /var/lib/pgbackrest
        chmod 750 /var/lib/pgbackrest
        mkdir -p /var/spool/pgbackrest
        chmod 750 /var/spool/pgbackrest

        # Throw warning if postgres user does not exist
        if ! id -u postgres > /dev/null 2>&1; then
            if [ ! -f /etc/pgbackrest.conf ]; then
                echo "*** WARNING: "
                echo "    The postgres user does not exist"
                echo "    /var/lib/pgbackrest, /var/log/pgbackrest and /var/spool/pgbackrest will be owned by root"
                echo "***"
            else
                echo "*** WARNING: "
                echo "    The postgres user does not exist"
                echo "    /var/lib/pgbackrest, /var/log/pgbackrest, /var/spool/pgbackrest and /etc/pgbackrest.conf will be owned by root"
                echo "***"
            fi
        else
            chown -R postgres /var/log/pgbackrest
            chown -R postgres /var/lib/pgbackrest
            chown -R postgres /var/spool/pgbackrest
            if [ -f /etc/pgbackrest.conf ]; then
                chown postgres /etc/pgbackrest.conf
            fi
        fi

        # Throw warning if postgres group does not exist
        if ! grep -q "postgres" /etc/group; then
            if [ ! -f /etc/pgbackrest.conf ]; then
                echo "*** WARNING: "
                echo "    The postgres group does not exist"
                echo "    /var/lib/pgbackrest, /var/log/pgbackrest and /var/spool/pgbackrest will be owned by root"
                echo "***"
            else
                echo "*** WARNING: "
                echo "    The postgres group does not exist"
                echo "    /var/lib/pgbackrest, /var/log/pgbackrest, /var/spool/pgbackrest and /etc/pgbackrest.conf will be owned by root"
                echo "***"
            fi
        else
            chgrp -R postgres /var/log/pgbackrest
            chgrp -R postgres /var/lib/pgbackrest
            chgrp -R postgres /var/spool/pgbackrest
            if [ -f /etc/pgbackrest.conf ]; then
                chgrp postgres /etc/pgbackrest.conf
            fi
        fi
    fi

    if [ -f /etc/pgbackrest.conf.rpmsave ]; then
        echo "*** NOTE: "
        echo "    There was a pgbackrest.conf.rpmsave found on the system.  During a previous uninstall your pgbackrest.conf was saved"
        echo "    to this file.  If you wish to use this saved config file please review and rename it to pgbackrest.conf"
        echo "***"
    fi

elif [ "$1" == "2" ]; then
    # UPGRADE
    # If we had a .9x conf file, save it before deletion
    if [ -f /etc/pg_backrest.conf ]; then
        %{__cp} /etc/pg_backrest.conf  /etc/pg_backrest.conf.old
    fi

    # Need to set ownership of pgbackrest.conf file to match what it was before
    if [ -f /etc/pgbackrest.conf ]; then
        chown --reference=/etc/pgbackrest.conf.crunchy.tmp /etc/pgbackrest.conf
        chgrp --reference=/etc/pgbackrest.conf.crunchy.tmp /etc/pgbackrest.conf
    fi

    # Now remove the temporary file created to maintain ownership
    rm -rf /etc/pgbackrest.conf.crunchy.tmp

fi

# This is needed to clean up the old way of doing upgrades and preserving ownerships
if [ -d /var/log/pgbackrest/pgbackrest.upgrade ] || [ -d /var/lib/pgbackrest/pgbackrest.upgrade ]; then
    rm -rf /var/log/pgbackrest/pgbackrest.upgrade
    rm -rf /var/lib/pgbackrest/pgbackrest.upgrade
    rm -rf /var/log/pgbackrest.upgrade
    rm -rf /var/lib/pgbackrest.upgrade
fi

%posttrans
#  In version 2.0.3 of backrest we no longer create or install the perl modules
#    nor the LIBC content so we need to clean these up from a previous install if
#    they exist

if [ -d %{PERL_LIB_DIR}/pgBackRest/Archive/Get ]; then

    # Need to check for the existence of the IO directory
    # Between 1.18 and 1.19 we changed the name from IO to Io and due to this logic
    #   The directory was never deleted and will cause an uninstall error

    if [ -d %{PERL_LIB_DIR}/pgBackRest/Protocol/IO ]; then
        rmdir %{PERL_LIB_DIR}/pgBackRest/Protocol/IO
    fi

    rmdir %{PERL_LIB_DIR}/pgBackRest/Archive/Get
    rmdir %{PERL_LIB_DIR}/pgBackRest/Archive/Push
    rmdir %{PERL_LIB_DIR}/pgBackRest/Archive
    rmdir %{PERL_LIB_DIR}/pgBackRest/Backup/Filter
    rmdir %{PERL_LIB_DIR}/pgBackRest/Backup
    rmdir %{PERL_LIB_DIR}/pgBackRest/Check
    rmdir %{PERL_LIB_DIR}/pgBackRest/Common/Http
    rmdir %{PERL_LIB_DIR}/pgBackRest/Common/Io
    rmdir %{PERL_LIB_DIR}/pgBackRest/Common
    rmdir %{PERL_LIB_DIR}/pgBackRest/Config
    rmdir %{PERL_LIB_DIR}/pgBackRest/Protocol/Base
    rmdir %{PERL_LIB_DIR}/pgBackRest/Protocol/Command
    rmdir %{PERL_LIB_DIR}/pgBackRest/Protocol/Local
    rmdir %{PERL_LIB_DIR}/pgBackRest/Protocol/Remote
    rmdir %{PERL_LIB_DIR}/pgBackRest/Protocol/Storage
    rmdir %{PERL_LIB_DIR}/pgBackRest/Protocol
    rmdir %{PERL_LIB_DIR}/pgBackRest/Storage/Cifs
    rmdir %{PERL_LIB_DIR}/pgBackRest/Storage/Filter
    rmdir %{PERL_LIB_DIR}/pgBackRest/Storage/Posix
    rmdir %{PERL_LIB_DIR}/pgBackRest/Storage/S3
    rmdir %{PERL_LIB_DIR}/pgBackRest/Storage
    rmdir %{PERL_LIB_DIR}/pgBackRest
    rmdir %{C_LIB_DIR}/pgBackRest
    rmdir %{C_LIB_DIR}/auto/pgBackRest/LibC
    rmdir %{C_LIB_DIR}/auto/pgBackRest
    rmdir %{C_LIB_DIR}/auto
fi

%postun
# If completely uninstalling, delete leftover directories
# This takes arguments:
#  - If "0" then it is an uninstall and we need to let them know we are not deleting their backups
#  - If "1" then we're uninstalling something before installing a new package and do nothing

if [ "$1" = "0" ]; then
    echo "=============================================================================="
    echo "NOTE: "
    echo "    Preserving the content in /var/log/pgbackrest, /var/spool/pgbackrest and"
    echo "    /var/lib/pgbackrest.  These directories use a lot of space so if you are sure"
    echo "    you don't need the backups any longer you should delete these directories."
    echo "=============================================================================="

fi

%clean
rm -rf ${RPM_BUILD_ROOT}

%files
%defattr(-,root,root,-)
# Include crunchy-backrest binary.
%{_bindir}/pgbackrest

# Include default configuration file.
%config(noreplace) /etc/pgbackrest.conf
%config(noreplace) %{_sysconfdir}/logrotate.d/%{name}

%doc LICENSE

%changelog
* Mon Mar 18 2019 Heath Lord <heath.lord@crunchydata.com>
- Fixing the formatting of all informational echos to the user.  

* Mon Mar 11 2019 Heath Lord <heath.lord@crunchydata.com>
- Make install of pgbackrest.conf conditional on whether they are using new config file

* Tue Oct 16 2018 John Harvey <john.harvey@crunchydata.com>
- Support version 2.06

* Wed Jun 20 2018 Heath Lord <heath.lord@crunchydata.com>
- Support version 2.03 and make install/uninstall procedures more sane

* Mon May 07 2018 John Harvey <john.harvey@crunchydata.com>
- Support version 2.02

* Tue Apr 17 2018 John Harvey <john.harvey@crunchydata.com>
- Make upgrading smarter space-wise

* Fri Jan 26 2018 John Harvey <john.harvey@crunchydata.com>
- Support version 1.27
- Fix upgrading chowning problem

* Tue Sep 05 2017 John Harvey <john.harvey@crunchydata.com>
- Support version 1.23

* Tue Feb 07 2017 Heath Lord <heath.lord@crunchydata.com>
- Added the support for having the C-library built and released

* Mon Feb 06 2017 Heath Lord <heath.lord@crunchydata.com>
- Removed Archive*.pm modules and added new Protocol pm files

* Thu Dec 22 2016 John Harvey <john.harvey@crunchydata.com>
- Update copyrights, move config file from %files to %config

* Wed Nov 09 2016 Heath Lord <heath.lord@crunchydata.com>
- Added Stanza.pm to the list of files for packaging

* Thu Aug 25 2016 Heath Lord <heath.lord@crunchydata.com>
- Added to pm files to packaging

* Wed Jun 1 2016 John Harvey <john.harvey@crunchydata.com>
- Support version 1.01

* Mon Apr 18 2016 John Harvey <john.harvey@crunchydata.com>
- Support version 1.00

* Mon Nov 23 2015 John Harvey <john.harvey@crunchydata.com>
- Updates to package dependencies in support of version 0.88.
- Updates to installed files in support of version 0.88.

* Wed Oct 14 2015 Adam Brightwell <adam.brightwell@crunchydata.com>
- Updates to package dependencies in support of version 0.85.
- Updates to installed files in support of version 0.85.

* Thu Aug 20 2015 Adam Brightwell <adam.brightwell@crunchydata.com>
- Updates to package dependencies in support of version 0.80.
- Updates to installed files in support of version 0.80.

* Thu Jul 02 2015 Adam Brightwell <adam.brightwell@crunchydata.com>
- Initial version.

